@@ -3,6 +3,8 @@ from .orthanc_folder_importer import *
 from .orthanc_monitor import OrthancMonitor
 from .orthanc_test_db_populator import OrthancTestDbPopulator
 from .pacs_migrator import PacsMigrator
+from .orthanc_comparator import OrthancComparator
+from .scheduler import Scheduler
 
 
 # Set default logging handler to avoid "No handler found" warnings.
