@@ -2,7 +2,7 @@ import os
 import logging
 import argparse
 import datetime
-from scheduler import Scheduler
+from .scheduler import Scheduler
 from orthanc_api_client import helpers
 
 from orthanc_api_client import OrthancApiClient
@@ -17,7 +17,11 @@ class OrthancComparator:
                  to_study_date: datetime.date,  # End date
                  modality: str,  # modality as configured in Orthanc (alias)
                  level: str = 'Series',
-                 scheduler: Scheduler = None
+                 scheduler: Scheduler = None,
+                 transfer_missing_to_modality: bool = False,
+                 ignore_missing_from_orthanc: bool = False,
+                 retrieve_missing_from_orthanc: bool = False,
+                 ignore_missing_on_modality: bool = False
                  ):
 
         if level not in ["Study", "Series", "Instance"]:
@@ -29,6 +33,10 @@ class OrthancComparator:
         self._to_study_date = to_study_date
         self._scheduler = scheduler
         self._level = level
+        self._transfer_missing_to_modality = transfer_missing_to_modality
+        self._ignore_missing_from_orthanc = ignore_missing_from_orthanc
+        self._retrieve_missing_from_orthanc = retrieve_missing_from_orthanc
+        self._ignore_missing_on_modality = ignore_missing_on_modality
 
     def execute(self):
 
@@ -75,22 +83,39 @@ class OrthancComparator:
             print(f"found {len(local_studies)} studies on both side")
 
         for local_study in local_studies:
+
             remote_match = [r for r in remote_studies if r.dicom_id == local_study.dicom_id]
             study_summary = f"{local_study.patient_main_dicom_tags.get('PatientID')} - {local_study.patient_main_dicom_tags.get('PatientName')} - {local_study.main_dicom_tags.get('StudyDescription')}"
-            if len(remote_match) == 0:
-                print(f"WARNING {str(current_date)}, study missing from modality: {study_summary}")
+
+            if len(remote_match) == 0 and not self._ignore_missing_on_modality:
+                print(f"WARNING {str(current_date)}, study missing on modality: {study_summary}")
+                if self._transfer_missing_to_modality:
+                    print(f"WARNING {str(current_date)}, transferring study to modality: {study_summary}")
+                    self._api_client.modalities.store(
+                        modality=self._modality,
+                        resources_ids=local_study.orthanc_id,
+                        synchronous=True
+                    )
             elif len(remote_match) > 1:
                 print(f"WARNING {str(current_date)}, study found multiple times on modality: {study_summary}")
-            else:
+            elif len(remote_match) == 1:
                 if self._level in ['Series']:
                     self.compare_study(orthanc_id=local_study.orthanc_id, dicom_id=local_study.dicom_id, study_summary=study_summary)
 
-        for remote_study in remote_studies:
-            local_match = [l for l in local_studies if l.dicom_id == remote_study.dicom_id]
-            if len(local_match) == 0:
-                print(f"WARNING {str(current_date)}, study missing from Orthanc: {remote_study.tags.get('PatientID')} - {remote_study.tags.get('PatientName')} - {remote_study.tags.get('StudyDescription')}")
-            elif len(local_match) > 1:
-                print(f"WARNING {str(current_date)}, study found multiple times on Orthanc: {remote_study.tags.get('PatientID')} - {remote_study.tags.get('PatientName')} - {remote_study.tags.get('StudyDescription')}")
+        if not self._ignore_missing_from_orthanc:
+            for remote_study in remote_studies:
+                local_match = [l for l in local_studies if l.dicom_id == remote_study.dicom_id]
+                if len(local_match) == 0:
+                    print(f"WARNING {str(current_date)}, study missing from Orthanc: {remote_study.tags.get('PatientID')} - {remote_study.tags.get('PatientName')} - {remote_study.tags.get('StudyDescription')}")
+                    if self._retrieve_missing_from_orthanc:
+                        print(f"WARNING {str(current_date)}, retrieving missing study from Orthanc: {remote_study.tags.get('PatientID')} - {remote_study.tags.get('PatientName')} - {remote_study.tags.get('StudyDescription')}")
+                        self._api_client.modalities.move_study(
+                            from_modality=self._modality,
+                            dicom_id=remote_study.dicom_id
+                        )
+                elif len(local_match) > 1:
+                    print(f"WARNING {str(current_date)}, study found multiple times on Orthanc: {remote_study.tags.get('PatientID')} - {remote_study.tags.get('PatientName')} - {remote_study.tags.get('StudyDescription')}")
+                # elif self._ignore_missing_on_modality: # in this case only, study comparison has not been performed above -> do it now
 
     def compare_study(self, orthanc_id: str, dicom_id: str, study_summary: str):
         if self._scheduler:
@@ -112,21 +137,35 @@ class OrthancComparator:
         for local_serie in local_series:
             local_dicom_id = local_serie.get('MainDicomTags').get('SeriesInstanceUID')
             remote_match = [r for r in remote_series if r.dicom_id == local_dicom_id]
-            if len(remote_match) == 0:
+            if len(remote_match) == 0 and not self._ignore_missing_on_modality:
                 print(f"WARNING STUDY {study_summary}, series missing from modality: {local_dicom_id}")
+                if self._transfer_missing_to_modality:
+                    print(f"WARNING STUDY {study_summary}, transferring series to modality: {local_dicom_id}")
+                    self._api_client.modalities.store(
+                        modality=self._modality,
+                        resources_ids=local_serie.get('ID'),
+                        synchronous=True
+                    )
             elif len(remote_match) > 1:
                 print(f"WARNING STUDY {study_summary}, series found multiple times on modality: {local_dicom_id}")
-            else:
+            elif len(remote_match) == 1:
                 series_summary = f"{local_dicom_id} (from STUDY {study_summary})"
                 if self._level in ['Instance']:
                     self.compare_series(orthanc_id=local_serie.get('ID'), dicom_id=local_dicom_id, series_summary=series_summary)
 
-        for remote_serie in remote_series:
-            local_match = [l for l in local_series if l.get('MainDicomTags').get('SeriesInstanceUID') == remote_serie.dicom_id]
-            if len(local_match) == 0:
-                print(f"WARNING STUDY {dicom_id}, series missing from Orthanc: {remote_serie.dicom_id}")
-            elif len(local_match) > 1:
-                print(f"WARNING STUDY {dicom_id}, series found multiple times on Orthanc: {remote_serie.dicom_id}")
+        if not self._ignore_missing_from_orthanc:
+            for remote_serie in remote_series:
+                local_match = [l for l in local_series if l.get('MainDicomTags').get('SeriesInstanceUID') == remote_serie.dicom_id]
+                if len(local_match) == 0:
+                    print(f"WARNING STUDY {dicom_id}, series missing from Orthanc: {remote_serie.dicom_id}")
+                    if self._retrieve_missing_from_orthanc:
+                        print(f"WARNING STUDY {dicom_id}, retrieving missing series from Orthanc: {remote_serie.dicom_id}")
+                        self._api_client.modalities.move_series(
+                            from_modality=self._modality,
+                            dicom_id=remote_serie.dicom_id
+                        )
+                elif len(local_match) > 1:
+                    print(f"WARNING STUDY {dicom_id}, series found multiple times on Orthanc: {remote_serie.dicom_id}")
 
 
     def compare_series(self, orthanc_id: str, dicom_id: str, series_summary: str):
@@ -146,17 +185,32 @@ class OrthancComparator:
         for local_instance in local_instances:
             local_dicom_id = local_instance.get('MainDicomTags').get('SOPInstanceUID')
             remote_match = [r for r in remote_instances if r.dicom_id == local_dicom_id]
-            if len(remote_match) == 0:
+
+            if len(remote_match) == 0 and not self._ignore_missing_on_modality:
                 print(f"WARNING SERIES {series_summary}, instance missing from modality: {local_dicom_id}")
+                if self._transfer_missing_to_modality:
+                    print(f"WARNING SERIES {series_summary}, transferring instance to modality: {local_dicom_id}")
+                    self._api_client.modalities.store(
+                        modality=self._modality,
+                        resources_ids=local_instance.get('ID'),
+                        synchronous=True
+                    )
             elif len(remote_match) > 1:
                 print(f"WARNING SERIES {series_summary}, instance found multiple times on modality: {local_dicom_id}")
 
-        for remote_instance in remote_instances:
-            local_match = [l for l in local_instances if l.get('MainDicomTags').get('SOPInstanceUID') == remote_instance.dicom_id]
-            if len(local_match) == 0:
-                print(f"WARNING SERIES {series_summary}, instance missing from Orthanc: {remote_instance.dicom_id}")
-            elif len(local_match) > 1:
-                print(f"WARNING SERIES {series_summary}, instance found multiple times on Orthanc: {remote_instance.dicom_id}")
+        if not self._ignore_missing_from_orthanc:
+            for remote_instance in remote_instances:
+                local_match = [l for l in local_instances if l.get('MainDicomTags').get('SOPInstanceUID') == remote_instance.dicom_id]
+                if len(local_match) == 0:
+                    print(f"WARNING SERIES {series_summary}, instance missing from Orthanc: {remote_instance.dicom_id}")
+                    if self._retrieve_missing_from_orthanc:
+                        print(f"WARNING SERIES {series_summary}, retrieving instance missing from Orthanc: {remote_instance.dicom_id}")
+                        self._api_client.modalities.move_instance(
+                            from_modality=self._modality,
+                            dicom_id=remote_instance.dicom_id
+                        )
+                elif len(local_match) > 1:
+                    print(f"WARNING SERIES {series_summary}, instance found multiple times on Orthanc: {remote_instance.dicom_id}")
 
 
 
@@ -180,8 +234,10 @@ if __name__ == '__main__':
     parser.add_argument('--from_study_date', type=str, required=True, help='From Study Date (format 20190225)')
     parser.add_argument('--to_study_date', type=str, required=True, help='To Study Date (format 20190225)')
     parser.add_argument('--level', type=str, default='Series', help='Compare resources up to Study/Series/Instance level')
-
-    # TODO parser.add_argument('--transfer_missing_to_modality', default=False, action='store_true', help='Transfer missing resources from Orthanc to the remote modality')
+    parser.add_argument('--transfer_missing_to_modality', default=False, action='store_true', help='Transfer missing resources from Orthanc to the remote modality')
+    parser.add_argument('--ignore_missing_from_orthanc', default=False, action='store_true', help="Don't generate a warning if resources are missing from Orthanc")
+    parser.add_argument('--retrieve_missing_from_orthanc', default=False, action='store_true', help='Retrieve missing resources to Orthanc from the remote modality')
+    parser.add_argument('--ignore_missing_on_modality', default=False, action='store_true', help="Don't generate a warning if resources are missing on the remote modality")
     # TODO parser.add_argument('--retrieve_missing_in_orthanc', default=False, action='store_true', help='Retrieve missing resources from remote modality into Orthanc')
     Scheduler.add_parser_arguments(parser)
 
@@ -194,6 +250,10 @@ if __name__ == '__main__':
     level = os.environ.get("LEVEL", args.level)
     from_study_date = helpers.from_dicom_date(os.environ.get("FROM_STUDY_DATE", args.from_study_date))
     to_study_date = helpers.from_dicom_date(os.environ.get("TO_STUDY_DATE", args.to_study_date))
+    transfer_missing_to_modality = os.environ.get("TRANSFER_MISSING_TO_MODALITY", args.transfer_missing_to_modality)
+    ignore_missing_from_orthanc = os.environ.get("IGNORE_MISSING_FROM_ORTHANC", args.ignore_missing_from_orthanc)
+    retrieve_missing_from_orthanc = os.environ.get("RETRIEVE_MISSING_FROM_ORTHANC", args.retrieve_missing_from_orthanc)
+    ignore_missing_on_modality = os.environ.get("IGNORE_MISSING_ON_MODALITY", args.ignore_missing_on_modality)
 
     scheduler = Scheduler.create_from_args_and_env_var(args)
 
@@ -203,7 +263,11 @@ if __name__ == '__main__':
         from_study_date=from_study_date,
         to_study_date=to_study_date,
         level=level,
-        scheduler=scheduler
+        scheduler=scheduler,
+        transfer_missing_to_modality=transfer_missing_to_modality,
+        ignore_missing_from_orthanc=ignore_missing_from_orthanc,
+        retrieve_missing_from_orthanc=retrieve_missing_from_orthanc,
+        ignore_missing_on_modality=ignore_missing_on_modality
     )
 
     comparator.execute()
